/* ******************************************************************************* */
/*                                                                                 */
/*                        TTPaint 256 V 1.0                                        */
/*                                                                                 */
/*            Geschrieben von Carsten & Henrik Behrens                             */
/*                                                                                 */
/*            Voraussetzung     :   ATARI TT mit Farbmonitor                       */
/*            Empfohlen         :   NVDI 2.0                                       */
/*            Programmiersprache:   Pure C                                         */
/*                                                                                 */
/*                                                                                 */
/*            Ein Malprogramm fr den 320 * 480 * 8 Modus (256 Farben)             */
/*                                                                                 */
/*            Markt & Technik     ST-Magazin 1992                                  */
/*                                                                                 */
/* ******************************************************************************* */

/* ******************************************************************************* */
/*     Teil 2: Grafische Grundfunktionen, Men, Farben, Malen                      */

/* ******************************************************************************* */
/* Funktionsname        : Change_Screen                                            */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Vertauscht die Bildschirmseiten (XBIOS)                  */
/* ******************************************************************************* */

void Change_Screen(void)
{
	long *dummy;				/* Vertauschen */
	dummy = Screen1;
	Screen1 = Screen2;
	Screen2 = dummy;
	Setscreen (Screen1, Screen1, -1);	/* Setzen */
}

/* ******************************************************************************* */
/* Funktionsname        : TTPaint                                                  */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Menverwaltung                                           */
/* ******************************************************************************* */

void TTPaint (void)
{
	Init_TTPaint ();
	Get_User_Interaction();
}

/* ******************************************************************************* */
/* Funktionsname        : Erase_Pic                                                */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Lscht das Arbeitsblatt                                  */
/* ******************************************************************************* */

void Erase_Pic(void)
{
	int Result;
	
	while (MouseK == 1)		/* Mausknopf noch gedrckt? */
		vq_mouse (hwdle, &MouseK, &MouseX, &MouseY);
	Result = form_alert(2, [3][Bild lschen?][Ja|Nein]);
	if (Result == 1)
	{
		Change_Screen ();		/* Arbeitsblatt */
		v_clrwk (handle);		/* lschen */
		Change_Screen ();		/* Toolbox */
	}
}./* ******************************************************************************* */
/* Funktionsname        : Quit                                                     */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Setzt Terminierungsflag                                  */
/* ******************************************************************************* */

void Quit(void)
{
	int Result;

	while (MouseK == 1)
		vq_mouse (handle, &MouseK, &MouseX, &MouseY);

	Result = form_alert (2, [3][Programm verlassen][Ja|Nein]);
	if (Result == 1)
		Ende = 1;				/* Terminierungsflag */
}

/* ******************************************************************************* */
/* Funktionsname        : Free_Hand                                                */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Freihandzeichnen mit der Maus                            */
/* ******************************************************************************* */

void Free_Hand(void)
{
	To_Paper();				/* Zum Arbeitsblatt */
	while (MouseK != 2)		/* Rechte Taste bed. Ende */
	{
		vq_mouse(handle, &MouseK, &MouseX, &MouseY);
		while (MouseK == 1)	/* Malen, solange linke Taste */
		{
			vq_mouse (handle, &MouseK, &MouseX, &MouseY);
			Point (MouseX, MouseY, L_Width, Current_color);

		}
	}
	To_Menu();					/* Zur Toolbox */
}

/* ******************************************************************************* */
/* Funktionsname        : Init_TTPaint                                             */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Grundlegende Initialisierung                             */
/* ******************************************************************************* */

void Init_TTPaint(void)
{
	F_toggle = 0;				/* Flag leer/gefllt */
	Current_color = 1;		/* Aktuelle Zeichenfarbe */
	Oldcolor = 0;
	L_Width = 0;				/* Linienstrke */
	Airadius = 16;				/* Sprhradius */
	OldCode = Code = 4;		/* Toolcode auf Freihand */

	Init_Palette();

	Setscreen (Screen2, Screen2, -1);	/* Arbeitsblatt lschen */
	v_clrwk(handle);
	Setscreen (Screen1, Screen1, -1);	/* Auf Toolbox schalten */
	Set_Clipping_Rect(0, 0, 319, 479, 1);
	v_clrwk (handle);			/* und lschen */

	vsf_interior (handle, 1);	/* GEM-Parameter setzen */
	vswr_mode (handle, 1);
	vsf_style (handle, 8);

	v_hide_c (handle);
	Draw_Menue();				/* Menbild aufbauen */
	v_show_c (handle, 0);
	graf_mouse (ARROW, NULL);

	Mark_Icon (Code);
}

/* ******************************************************************************* */
/* Funktionsname        : Draw_Menue                                               */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet die Toolbox und Palette                         */
/* ******************************************************************************* */

void Draw_Menue(void)
{
	int i, File;
	
	Box (0, 0, 319, 479, 242, 1);	/* Hintergrund grau */

	File = Fopen(TITLE,FO_READ);	/* Titel laden, falls vorh. */
	if(File >= 0)
	{
		Fread(File, 32000L, Screen1);
		Fclose(File);
	}

	Box (Mx + 10, My + 10, Mx + 170, My + 170, 241, 1);	/* Schatten Toolbox */
	Box (Mx, My, Mx + 160, My + 160, 243, 1);	/* Toolbox */
	Box (210, 120, 309, 280, 241, 1);	/* Farbmischer */
	Box (200, 110, 299, 270, 243, 1);

	for (i = Mx; i <= Mx + 160; i += 40);	/* Toolboxfelder vert. */
		Line (i, My, i, My + 160, 244);

	for (i = My; i <= My + 160; I += 40)	/* Toolboxfelder hor. */
		Line (Mx, i, Mx + 160, i, 244);

	Box (200, 110, 300, 270, 244, 0);

	Diskette (Mx + 10, My + 5);	/* Bild laden */
	Line (Mx + 20, My + 27, Mx + 20, My + 37, 244);
	Line (Mx + 17, My + 34, Mx + 20, My - 37, 244);
	Line (Mx + 20, MY + 37, Mx + 23, My + 34, 244);

	Diskette (Mx + 50, MY + 5);	/* Bild speichern */
	Line (Mx + 60, My + 27, Mx + 60, My + 37, 244);
	Line (Mx + 57, My + 30, Mx + 60, My + 27, 244);
	Line (Mx + 60, My + 27, Mx + 63, MY + 30, 244);

	Box (Mx + 90, My + 10, Mx + 110, My + 30, 247, 1);	/*Lschen */
	Line (Mx + 92, My + 7, Mx + 108, My + 33, 245);
	Line (Mx + 92, My + 33, Mx + 108, My + 7, 245);

	Box (Mx + 123, My +  2, Mx + 127, My +  3, 244, 1);	/* Ende */
	Box (Mx + 129, My +  2, Mx + 134, My +  3, 244, 1);
	Box (Mx + 136, My +  2, Mx + 141, My +  3, 244, 1);
	Box (Mx + 143, My +  2, Mx + 149, My +  3, 244, 1);
	Box (Mx + 121, My +  5, Mx + 159, My + 39, 247, 1);
	Box (Mx + 122, My +  7, Mx + 126, My + 11, 243, 1);
	Box (Mx + 122, My + 13, Mx + 126, My + 17, 243, 1);
	Box (Mx + 130, My +  9, Mx + 155, My + 30, 243, 1);

	Line (Mx +  5, My + 65, Mx + 10, My + 55, 244);	/* Freihand */
	Line (Mx + 10, My + 55, Mx + 15, My + 50, 244);
	Line (Mx + 15, My + 50, Mx + 20, My + 55, 244);
	Line (Mx + 20, My + 55, Mx + 25, My + 65, 244);
	Line (Mx + 25, My + 65, Mx + 30, My + 70, 244);
	Line (Mx + 30, My + 70, Mx + 35, My + 65, 244);
	Line (Mx + 50, My + 70, Mx + 70, My + 50, 244);	/* Linie */

	Toggleicons();

	Box (Mx + 45, My + 85, Mx + 50, My + 92, 244, 0);	/*Fll 1/0 */
	Ellipse (Mx + 47, My + 101, 3, 3, 244, 0);
	
	Parray[O] = Mx +  47;
	Parray[1] = My + 109;
	Parray[2] = Mx +  50;
	Parray[3] = My + 115;
	Parray[4] = Mx +  45;
	ParraY151 = My + 115;
	Polygon (Parray, 3, 244, 0);

	Box (Mx + 70, My + 85, Mx + 75, My + 92, 244, 1);
	Ellipse (Mx + 72, My + 101, 3, 3, 244, 1);

	Parray[O] = Mx +  72;
	Parray[1] = My + 109;
	Parray[2] = Mx +  75;
	Parray[3] = My + 115;
	Parray[4] = Mx +  70;
	Parray[5] = My + 115;
	Polygon (Parray, 3, 244, 1);

	Line (Mx + 54, My + 101, Mx + 66, My + 101, 244);
	Line (Mx + 54, My + 101, Mx + 57, My +  98, 244);
	Line (Mx + 54, My + 101, Mx + 57, My + 104, 244);
	Line (Mx + 66, My + 101, Mx + 63, My +  98, 244);
	Line (Mx + 66, My + 101, Mx + 63, My + 104, 244);

	Text (ABCD, Mx + 84, My + 108, 244);	/* Text */

	Parray[O] = Mx + 127;		/* Fllen */
	Parray[1] = My +  95;
	Parray[2] = Mx + 141;
	Parray[3] = My +  95;
	Parray[4] = Mx + 134;
	Parray[5] = My + 102;
	Polygon (Parray, 3, 245, 1);

	Parray[O] = Mx + 127;
	Parray[1] = My +  95;
	Parray[2] = Mx + 134;
	Parray[3] = My +  88;
	Parray[4] = Mx + 141;
	Parray[5] = My +  95;
	Parray[6] = Mx + 134;
	Parray[7] = My + 102;
	Polygon (Parray, 4, 244, 0);

	Line (Mx + 141, My +  95, Mx + 141, My + 109, 245);
	Line (Mx + 138, My + 110, Mx + 144, My + 110, 245);
	Line (Mx + 133, My + 111, Mx + 149, My + 111, 245);

	Tin (Mx, My);				/* Spray */

	Ellipse (Mx + 54, My + 135, 8, 8, 244);	/* Lupe */
	Line    (Mx + 60, My + 141, Mx + 74, My + 155, 244);

	Ellipse (Mx + 90, My + 130, 2, 2, 244, 1);	/* Pinsel */
	Ellipse (Mx + 90, My + 140, 3, 3, 244, 1);
	Ellipse (Mx + 90, My + 150, 4, 4, 244, 1);
	Line (Mx + 110, My + 125, Mx + 117, My + 125, 244);
	Line (Mx + 117, My + 125, Mx + 117, My + 140, 244);
	Line (Mx + 117, My + 140, Mx + 110, My + 14o, 244);
	Line (Mx + 110, My + 140, Mx + 110, My + 150, 244);
	Ellipse (Mx + 110, My + 154, 2, 2, 244, 0);

	Tin (Mx + 110, My);		/* Sprhradius */
	Line (Mx + 150, My + 125, Mx + 157, My + 125, 244);
	Line (Mx + 157, My + 125, Mx + 157, My + 140, 244);
	Line (Mx + 157, My + 140, Mx + 150, My + 140, 244);
	Line (Mx + 150, My + 140, Mx + 150, My + 150, 244);
	Ellipse (Mx + 150, My + 154, 2, 2, 244, 0);

	Box (212, 232, 243, 263, Current_color, 1);	/* Farbmischer */
	Box (256, 232, 287, 263, 242, 1);
	Box (256, 232, 287, 263, 244, 0);
	Text (INT, 260, 253, 244);

	Text (R, 217, 125, 244);
	Text (G, 246, 125, 244);
	Text (B, 275, 125, 244);

	for (i = 144; i < 211; i += 4)
	{
		if (i % 8)
		{
			Line (211, i, 230, i, 244);
			Line (240, i, 259, i, 244);
			Line (269, i, 288, i, 244);
		}
		else
		{
			Line(210, i, 231, i, 244);
			Line(239, i, 260, i, 244);
			Line(268, i, 289, i, 244);
		}
	}

	Show_Colors (16, 320);	/* Farbpalette */
}

/* ******************************************************************************* */
/* Funktionsname        : Get_User_Interaction                                     */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: berwacht Benutzungsoberflche                           */
/* ******************************************************************************* */

void Get_User_Interaction(void)
{
	while (!Ende)				/* Hauptschleife */
	{
		Check_Colorbox();
		Check_Toolbox();
		Use_Tool();
	}
}./* ******************************************************************************* */
/* Funktionsname        : Check_Colorbox                                           */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: berprft, ob neue Zeichenfarbe gewhlt wurde            */
/* ******************************************************************************* */

void Check_Colorbox (void)
{
	vq_mouse (handle, &MouseK, &MouseX, &MouseY);	/* Maus */
	Current_color = Get_Colorindex (16, 320, 212, 232, 1);
	if (Oldcolor != Currentcolor)	/* Neue Farbe gewhlt? */
	{							/* Ja, Slider aktualisieren */
		vq_color (handle, Currentcolor, 0, Current_RGB);
		Draw_Slider (213, 135, Current_RGB[0]);
		Draw_Slider (242, 135, Current_RGB[1]);
		Draw_Slider (271, 135, Current_RGB[2]);
		Oldcolor = Currentcolor;	/* Neue Farbe merken */
	}
}

/* ******************************************************************************* */
/* Funktionsname        : Check_Toolbox                                            */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: berprft, ob neues Tool gewhlt wurde                   */
/* ******************************************************************************* */

void Check_Toolbox (void)
{
	if(MouseK == 1)			/* Linke Maustaste gedrckt? */
	{
		if ((MouseX > Mx) && (MouseX < Mx + 160) &&
		    (MouseY > My) && (MouseY < My + 160));	/* Zeiger in der Toolbox? */
		{
			MouseX - = Mx;		/* Code des Tools berechnen */
			MouseY - = My;
			MouseX / = 40;
			MouseY / = 40;
			Code  = (MouseY << 2);
			Code != MouseX;
			if (Code != OldCode)	/* Neues Tool? */
			{
				Mark_Icon (Code);	/* Neues Tool hervorheben */
				Mark_Icon (OldCode);	/* Altes Tool restaurieren */
				OldCode = Code;	/* Neuen Code merken */
			}

			switch (Code)		/* Verwaltungstools sofort aufrufen */
			{
				case  0:	Load_Pic();  break;
				case  1:	Save_Pic();  break;
				case  2:	Erase_Pic(); break;
				case  3:	Quit();      break;
				case  9:	Swap_Ft();   break;
				case 14:	L_Width = Set_LineWidth(); break;
				case 15:	Airadius = Set_Airbrush(); break;
			}
		}
		else					/* Wenn nicht Toolbox, viell. INT-Button */
			Check_INT;
	}
}./* ******************************************************************************* */
/* Funktionsname        : Use_Tool                                                 */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Aktuelles Tool anwenden                                  */
/* ******************************************************************************* */

void Use_Tool(void)
{
	if (MouseK == 2)			/* Rechte Maustaste ==> Arbeitsblatt */
		switch (Code)			/* Tool bereitstellen */
		{
			case 4:	Free_Hand(); break;
			case 5:	Free_Line(); break;
			case 6:	Free_Box(); break;
			case 7:	Free_Ellipse(); break;
			case 8:	Free_Poly(); break;
			case 10:	Free_Text(); break;
			case 11:	Free_Fill(); break;
			case 12:	Spray(); break;
			case 13:	Zoom(); break;
		}
}

/* ******************************************************************************* */
/* Funktionsname        : Toggleicons                                              */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet vernderliche Tools                             */

void Toggleicons (void)
{
	Box (Mx +  81, My + 41, Mx + 119, My +  79, 243, 1);
	Box (Mx + 121, My + 41, Mx + 159, My +  79, 243, 1);
	Box (Mx +   1, My + 81, Mx +  39, My + 119, 243, 1);
	Box (Mx +  90, My + 70, Mx + 110, My +  50, 244, F_toggle);
	Ellipse(Mx + 140, My + 60, 15, 10, 244, F_toggle);

	Parray[0] = Mx + 10;
	Parray[1] = My + 20 + 80;
	Parray[2] = Mx + 15;
	Parray[3] = My + 10 + 80;
	Parray[4] = Mx + 30;
	Parray[5] = My + 15 + 80;
	Parray[6] = Mx + 30;
	Parray[7] = My + 20 + 80;
	Parray[8] = Mx + 20;
	Parray[9] = My + 30 + 80;
	Parray[10] = Mx + 10;
	Parray[11] = My + 20 + 80;
	Polygon(Parray, 6, 244, F_toggle);
}

/* ******************************************************************************* */
/* Funktionsname        : Diskette                                                 */
/* bergabeparameter    : Bildschirmkoordinaten                                    */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Diskettensymbol an x,y                          */
/* ******************************************************************************* */

void Diskette(int x, int y)
{
	Box (x,      y,      x + 20, y + 20, 246, 1);
	Box (x +  5, y + 20, x + 15, y + 14, 242, 1);
	Box (x + 17, y +  2, x + 19, y +  4, 244, 1);
	Box (x +  4, y,      x + 16, y + 12, 247, 1);
}./* ******************************************************************************* */
/* Funktionsname        : Tin                                                      */
/* bergabeparameter    : Bildschirmkoordinaten                                    */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Sprhdosensymbol an x,y                         */
/* ******************************************************************************* */

void Tin (int x, int y)
{
	Box (x + 13, y + 138, x + 23, y + 155, 246, 1);
	Box (x + 17, y + 133, x + 19, y + 137, 244, 1);
	Box (x + 15, y + 138, x + 17, y + 155, 247, 1);

	Line (x + 20, y + 135, x + 30, y + 129, 245);
	Line (x + 20, y + 135, x + 30, y + 132, 245),
	Line (x + 20, y + 135, x + 30, Y + 135, 245);
	Line (x + 20, y + 135, x + 30, y + 138, 245);
	Line (x + 20, y + 135, x + 30, y + 141, 245);
}

/* ******************************************************************************* */
/* Funktionsname        : Mark_Icon                                                */
/* bergabeparameter    : Toolcode                                                 */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Invertiert angewhltes Tool                              */
/* ******************************************************************************* */

void Mark_Icon(int Code)
{
	int pxyarray[4];
	int x, y;

	x = Mx + (Code & 0x3) * 40;	/* Koordinaten aus Code */
	y = My + ((Code & Oxfffc) >> 2) * 40;	/* berechnen */

	vsl_color (handle, 246);
	vsf_color (handle, 246);
	vsf_interior (handle, 1);
	vsf_perimeter (handle, 1);
	vswr_mode (handle, 3);	/* Mode XOR setzen */

	pxyarray[0] = x;
	Pxyarray[1] = y;
	pxyarray[2] = x + 40;
	pxyarray[3] = y + 40;

	v_hide_c (handle);
	v_bar (handle, pxyarray);	/* Tool behandeln */
	v_show_c(hmdle, 1);
}

/* ******************************************************************************* */
/* Funktionsname        : To_Menue                                                 */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Schaltet vom Arbeitsblatt zur Toolbox                    */
/* ******************************************************************************* */

void To_Menue(void)
{
	while (MouseK == 2)		/* Rechte Maustaste noch gedrckt? */
		vq_mouse (handle, &MouseK, &MouseX, &MouseY);
	
	Change_Screen();			/* Zur Toolbox */
	graf_mouse (ARROW, 0);	/* Mauszeiger wird Pfeil */
}./* ******************************************************************************* */
/* Funktionsname        : To_Paper                                                 */
/* bergabeparameter    :                                                          */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Schaltet von Toolbox zum Arbeitsblatt                    */
/* ******************************************************************************* */

void To_Paper(void)
{
	vswr_mode (handle, 1);	/* Normaler Schreibmodus */

	while (MouseK == 2)
		vq_mouse (handle, &MouseK, &MouseX, &Mousey);

	Change_Screen();			/* Zum Arbeitsblatt */
	graf_mouse (THIN_CROSS, 0);	/* Mauszeiger wird Kreuz */
}

/* ******************************************************************************* */
/* Funktionsname        : Init_Palette                                             */
/* bergabeparameter    :                                                          */
/* Rckgabeparmeter     :                                                          */
/* Funktionsbeschreibung: Setzen der Toolboxfarben                                 */
/* ******************************************************************************* */

void Init_Palette(void)
{
	int i, n = 0;
	int colors [30] = {
							   0,    0,    0,
							1000, 1000, 1000,
							 600,  600,  600,
							 376,  376,  376,
							 563,  563,  563,
							 813,  813,  813,
							   0,    0,    0,
							1000, 1000,    0,
	 						   0,    0, 1000,
							   0,  500, 1000
						};
	int indices[10] = {
							  0,   1,   2, 241, 242,
							243, 244, 245, 246, 247
						};
	int rgb[3];

	for (i = 0; i < 10; i++)	/* Palette aus Tabelle setzen */
	{
		rgb[0] = colors[n++];
		rgb[1] = colors[n++];
		rgb[2] = colors[n++];
		vs_color (handle, indices[i], rgb);
	}
}./* ******************************************************************************* */
/* Funktionsname        : Get_Palette                                              */
/* bergabeparameter    : Pointer auf Palettenpuffer                               */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Liest RGB-Werte aus GEM-Palette in Palettenpuffer        */
/* ******************************************************************************* */

void Get_Palette (int *Buffer)
{
	int i, idx = 0;
	int rgb[3];

	for (i = 0, i < 256; i++)	/* 256 Farben in Puffer */
	{
		vq_color (handle, i, 0, rgb);
		Buffer[idx++] = rgb[0];
		Buffer[idx++] = rgb[1];
		Buffer[idx++] = rgb[2];
	}
}

/* ******************************************************************************* */
/* Funktionsname        : Set_Palette                                              */
/* bergabeparameter    : Pointer auf Palettenpuffer                               */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Schreibt RGB-Werte aus Palettenpuffer in GEM-Palette     */
/* ******************************************************************************* */

void Set_Palette (int *Buffer)
{
	int i, idx = 0;
	int rgb[3];

	for (i = 0; i < 256; i++)	/* 256 Farben aus Puffer */
	{
		rgb[0] = Buffer[idx++];
		rgb[1] = Buffer[idx++];
		rgb[2] = Buffer[idx++];
	
		vs_color(handle, i, rgb);
	}
}

/* ******************************************************************************* */
/* Funktionsname        : Show_Colors                                              */
/* bergabeparameter    : Linke, obere Ecke des Farbkastens                        */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Farbkasten an x, y                              */
/* ******************************************************************************* */

void Show_Colors (int sx, int sy)
{
	int x, y;
	int color = 0;

	Box (sx + 9, sy + 9, sx + 298, sy + 146, 241, 1);
	Box (sx - 1, sy - 1, sx + 288, sy + 136, 244, 0);

	for (y = sy; y < sy + 136; y += 17)
		for (x = sx; x < sx + 288; x += 9)
		{
			Box (x, y, x + 8, y + 16, color, 1);
			color++;
		}
	
	x = (Current_color & 31) * 9 + sx;
	y = ((Current_color & 224) >> 5) * 17 + sy;

	Box (x, y, x + 8, y + 16, 244,0);
}./* ******************************************************************************* */
/* Funktionsname        : Get_Colorindex                                           */
/* bergabeparmeter     : Linke obere Ecke des Farbkastens, Linke obere Ecke des   */
/*                        Farbindikators und Flag                                  */
/* Rckgabeparameter    : Farbindex                                                */
/* Funktionsbeschreibung: Auswahl einer Farbe aus dem Farbkasten und Hervorhebung  */
/*                        der gewhlten Farbe                                      */
/* ******************************************************************************* */

int Get_Colorindex (int sx, int sy, int bx, int by, int flag)
{
	int index, x, y;

								/* Mauszeiger im Farbkasten */
								/* und Mausknopf gedrckt ? */
	if ((MouseX >= sx) && (MouseX < (sx + 288)) &&
	    (MouseY >= sy) && (Mousey < (sy + 136)) && (MouseK == 1))

								/* Farbindex berechnen */
	{
		index = ((MouseX - sx) / 9) | (((MouseY - sy) / 17) << 5);
								/* Neue Farbe ? */
		if (Current_color != index)
		{
			if (flag)			/* gewhltes Farbfeld mar- */
								/* kieren und altes demar- */
								/* kieren */
			{
				x = (Current_color & 31) * 9 + sx;
				y = ((Currentcolor & 224) >> 5) * 17 + sy;
				Box (x, y, x + 8, y + 16, Current_color, 1);

				x = (index & 31) * 9 + sx;
				y = ((index & 224) >> 5) * 17 + sy;
				Box (x, y, x + 8, y + 16, 244, 0);
				Box (bx, by, bx + 31, by + 31, 244, 0);
				Box (bx, by, bx + 31, by + 31, index, 1);
			}
			retum(index);
		}
		return(Current_color);
}

/* ******************************************************************************* */
/* Funktionsname        : Plot                                                     */
/* bergabeparameter    : Bildschirmkoordinaten, Farbe                             */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreihung: Setzt Punkt an x,y in Farbe color                        */
/* ******************************************************************************* */

void Plot(int x, int y, int color)
{
	vsl_ends (handle, 0, 0);
	vsl_width (handle, 0);
	Line (x, y, x, y, color);	/* Pixel setzen */
}./* ******************************************************************************* */
/* Funktionsname        : Point                                                    */
/* bergabeparameter    : Bildschirmkoordinaten, Radius, Farbe                     */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Setzt Punkt mit Radius r an x, y in Farbe color          */
/* ******************************************************************************* */

void Point(int x, int y, int r, int color)
{
	if (r == 0)
		Plot(x, y, color);	/* Pixel */
	else
	{
		vsf_interior  (handle, 1);	/* Gefllter Kreis */
		vsf_perimeter (handle, 0);
		vsl_color     (handle, color);
		vsf_color     (handle, color);
		vswr_mode     (handle, 0);
		vsl_width     (handle, 1);
		v_hide_c      (handle);
		v_circle      (handle, x, y, r);
		v_show_c      (handle, 1);
		vsl_ends      (handle, 0, 0);
	}
}

/* ******************************************************************************* */
/* Funktionsname        : Line                                                     */
/* bergabeparameter    : Bildschimkoordinaten, Farbe                              */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Linie von PO nach P1 in Farbe color             */
/* ******************************************************************************* */

void Line (int xO, int yO, int x1, int y1, int color)
{
	int pxyarray[4];

	pxyarray[0] = xO;
	pxyarray[1] = yO;
	pxyarray[2] = x1;
	pxyarray[3] = y1;
	vsl_color (handle, color);
	v_hide_c (handle);
	v_pline (handle, 2, pxyarray);
	v_show_c (handle, 1);
	vsl_ends (handle, 0, 0);
}

/* ******************************************************************************* */
/* Punktionsname        : Box                                                      */
/* bergabeparameter    : Bildschimkoordinaten, Farbe, Flag                        */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet geflltes oder ungeflltes Rechteck             */
/*                        in Farbe color                                           */
/* ******************************************************************************* */

void Box (int xO, int yO, int xl, int yl, int color, int toggle)
{
	int pxyarray[4];

	vsl_color (handle, color);
	vsf_color (handle, color);
	vsf_interior (handle, toggle);
	vsf_perimeter (handle, !toggle);
	vswr_mode (handle, (!toggle + 1));

	pxyarray[O] = xO;
	pxyarray[1] = yO;
	pxyarray[2] = x1;
	pxyarray[3] = y1;
	v_hide_c (handle);
	v_bar (handle, pxyarray);
	v_show_c (handle, 1);
}

/* ******************************************************************************* */
/* Funktionsname        : Ellipse                                                  */
/* bergabeparameter    : Bildschirmkoordinaten, RadiusX, RadiusY, Farbe, Flag     */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Ellipse an x, y mit Radien rx, ry               */
/*                        in Farbe color gefllt oder ungefllt                    */
/* ******************************************************************************* */

void Ellipse (int x, int y, int rx, int ry, int color, int toggle)
{
	vsl_color (handle, color);
	vsf_color (handle, color);
	vsf_interior (handle, toggle);
	vsf_perimeter (handle, !toggle);
	vswr_mode (handle, (!toggle + 1));
	v_hide_c (handle);
	v_ellipse (handle, x, y, rx, ry);
	v_show_c (handle, 1);
}

/* ******************************************************************************* */
/* Funktionsname        : Polygon                                                  */
/* bergabeparameter    : Punktarray, Anzahl der Punkte, Farbe, Flag               */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Zeichnet Polygon in Farbe color                          */
/* ******************************************************************************* */

void Polygon (int *Parray, int count, int color, int toggle)
{
	vsf_interior (handle, toggle);
	vsf_perimeter (handle, !toggle);
	vsl_color (handle, color);
	vsf_color (handle, color);
	vsl_ends (handle, 0, 0);
	vsl_width (handle, 0);
	vswr_mode (handle, (!toggle + 1));
	Parray[count << 1] = Parray[O];	/* Polygon schlieen */
	Parray[(count << 1) + 1] = Parray[1];
	count++;
	v_hide_c (handle);
	v_fillarea (handle, count, Parray);
	v_show_c (handle, 1);
}

/* ******************************************************************************* */
/* Funktionsname        : Text                                                     */
/* bergabeparameter    : Zeichenkette, Bildschirmkoordinaten, Farbe               */
/* Rckgabeparameter    :                                                          */
/* Funktionsbeschreibung: Schreibt Zeichenkette an x, y in Farbe color             */
/* ******************************************************************************* */

void Text (char *String, int x, int y, int color)
{
	vsr_mode  (handle, 2);
	vst_color (handle, color);
	v_hide_c  (handle);
	v_gtext   (handle, x, y, String);
	v_show_c  (handle, 1);
}
